@@ -441,6 +441,9 @@ output_create(const struct Masscan *masscan, unsigned thread_index)
     case Output_Redis:
         out->funcs = &redis_output;
         break;
+    case Output_Hostonly:
+        out->funcs = &hostonly_output;
+        break;
     case Output_None:
         out->funcs = &null_output;
         break;
@@ -501,8 +504,8 @@ output_create(const struct Masscan *masscan, unsigned thread_index)
 static FILE *
 output_do_rotate(struct Output *out, int is_closing)
 {
-    const char *dir = out->rotate.directory;
-    const char *filename = out->filename;
+    const char *dir;
+    const char *filename;
     char *new_filename;
     size_t new_filename_size;
     struct tm tm;
@@ -512,15 +515,21 @@ output_do_rotate(struct Output *out, int is_closing)
     if (out == NULL || out->fp == NULL)
         return NULL;
 
+    dir = out->rotate.directory;
+    filename = out->filename;
+
     /* Make sure that all output has been flushed to the file */
     fflush(out->fp);
 
     /* Remove directory prefix from filename, we just want the root filename
      * to start with */
-    while (strchr(filename, '/') || strchr(filename, '\\')) {
+    while (strchr(filename, '/')) {
         filename = strchr(filename, '/');
         if (*filename == '/')
             filename++;
+    }
+
+    while (strchr(filename, '\\')) {
         filename = strchr(filename, '\\');
         if (*filename == '\\')
             filename++;
