@@ -11,16 +11,20 @@
  * Process an ARP packet received in response to an ARP-scan.
  ***************************************************************************/
 void
-handle_arp(struct Output *out, time_t timestamp, const unsigned char *px,
+arp_recv_response(struct Output *out, time_t timestamp, const unsigned char *px,
            unsigned length, struct PreprocessedInfo *parsed)
 {
-    unsigned ip_them;
+    ipaddress ip_them = parsed->src_ip;
 
     UNUSEDPARM(length);
     UNUSEDPARM(px);
 
-    ip_them = parsed->ip_src[0]<<24 | parsed->ip_src[1]<<16
-            | parsed->ip_src[2]<< 8 | parsed->ip_src[3]<<0;
+
+    LOG(3, "ARP %s = [%02X:%02X:%02X:%02X:%02X:%02X]\n",
+        ipaddress_fmt(ip_them).string,
+        parsed->mac_src[0], parsed->mac_src[1], parsed->mac_src[2], 
+        parsed->mac_src[3], parsed->mac_src[4], parsed->mac_src[5]);
+
 
     output_report_status(
                     out,
@@ -30,6 +34,7 @@ handle_arp(struct Output *out, time_t timestamp, const unsigned char *px,
                     0, /* ip proto */
                     0,
                     0,
-                    0);
+                    0,
+                    parsed->mac_src);
 
 }
